////////////////////////////////////////////////////////////////////////////////
// checkstyle: Checks Java source code for adherence to a set of rules.
// Copyright (C) 2001-2002  Oliver Burn
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package edu.virginia.cs.escjava;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;


/**
 * This class provides an abstraction to the ESC/Java extended static checker.
 * @author <a href="mailto:oliver@puppycrawl.com">Oliver Burn</a>
 * @author <a href="mailto:stephane.bailliez@wanadoo.fr">Stephane Bailliez</a>
 * @author lkuehne
 */
public class Checker
{
    /**
     * An AuditListener that maintains the number of errors.
     */
    private class ErrorCounter implements AuditListener
    {
        /** keeps track of the number of errors */
        private int mCount = 0;

        /** @see AuditListener */
        public void addError(AuditEvent aEvt)
        {
            if (SeverityLevel.ERROR.equals(aEvt.getSeverityLevel())) {
                mCount++;
            }
        }

        /** @see AuditListener */
        public void addException(AuditEvent aEvt, Throwable aThrowable)
        {
            mCount++;
        }

        /** @see AuditListener */
        public void auditStarted(AuditEvent aEvt)
        {
            mCount = 0;
        }

        /** @see AuditListener */
        public void fileStarted(AuditEvent aEvt)
        {
        }

        /** @see AuditListener */
        public void auditFinished(AuditEvent aEvt)
        {
        }

        /** @see AuditListener */
        public void fileFinished(AuditEvent aEvt)
        {
        }

        /**
         * @return the number of errors since audit started.
         */
        private int getCount()
        {
            return mCount;
        }
    }

    /** maintains error count */
    private final ErrorCounter mCounter = new ErrorCounter();

    /** vector of listeners */
    private final ArrayList mListeners = new ArrayList();

    /** vector of fileset checks */
    private final ArrayList mFileSetChecks = new ArrayList();

    /** class loader to resolve classes with. **/
    private ClassLoader mLoader =
            Thread.currentThread().getContextClassLoader();

    /** the basedir to strip off in filenames */
    private String mBasedir;

	private String escjavaExecutable;
	
    /**
     * The severity level of any violations found by submodules.
     * The value of this property is passed to submodules via
     * contextualize().
     *
     * Note: Since the Checker is merely a container for modules
     * it does not make sense to implement logging functionality
     * here. Consequently Checker does not extend AbstractViolationReporter,
     * leading to a bit of duplicated code for severity level setting.
     */
    private SeverityLevel mSeverityLevel = SeverityLevel.ERROR;

    /**
     * Creates a new <code>Checker</code> instance.
     * The instance needs to be contextualized and configured.
     *
     * @throws CheckstyleException if an error occurs
     */
    public Checker()
    {
    	escjavaExecutable = EscjavaPlugin.getDefault().getPreferenceStore().getString(EscjavaPlugin.ESCJAVA_LOCATION_PREFERENCE);
        addListener(mCounter);
    }


    /** Cleans up the object **/
    public void destroy()
    {
        mListeners.clear();
    }

    /**
     * Add the listener that will be used to receive events from the audit
     * @param aListener the nosy thing
     */
    public void addListener(AuditListener aListener)
    {
        mListeners.add(aListener);
    }

    /**
     * Processes a set of files with all FileSetChecks.
     * Once this is done, it is highly recommended to call for
     * the destroy method to close and remove the listeners.
     * @param aFiles the list of files to be audited.
     * @return the total number of errors found
     * @see #destroy()
     */
    public int process(File[] aFiles, String classpathString)
    {
        // Should actually be possible to pass all files to
        // escjava on one invocation. Worry about this some other time.
        // For now, just call escjava once for each source file.
        
        InputStream escjavaStream;
        EscjavaMessage[] messages;
        
        for (int i = 0; i < aFiles.length; i++) {
			try {
				String commandLine = escjavaExecutable;
				String fileArgument = aFiles[i].getName();
		
				String cmdArray[] = new String[2];
				cmdArray[0] = commandLine;
				cmdArray[1] = fileArgument;
				
				String envp[] = new String[1];
				envp[0] = "CLASSPATH=" + classpathString;
				System.out.println("envp is set to " + envp[0]);
//				System.out.println("starting escjava with " + cmdArray[0] + " and " + cmdArray[1] + " on " + aFiles[i].getParentFile().toString());
				Process proc = Runtime.getRuntime().exec(cmdArray, envp, aFiles[i].getParentFile());
				escjavaStream = proc.getInputStream();
				messages = parseOutput(escjavaStream);
				fireErrors(aFiles[i].getName(), messages);
			} catch (IOException e) {
				e.printStackTrace();
				continue;
			}
        }

        int errorCount = mCounter.getCount();
        return errorCount;
    }
    
    private EscjavaMessage[] parseOutput(InputStream istream) throws IOException
    {
    	BufferedReader in = new BufferedReader(new InputStreamReader(istream));
    	String outputLine;
    	Collection messages = new Vector();
    	boolean more = false;
    	String moreString = "";
    	EscjavaMessage currentMessage = null;
    	int counter = 0;
    	
    	while ((outputLine = in.readLine()) != null)
    	{
    		System.out.println("parsing: " + outputLine);
    		// Gather all of the information after an escjava warning message
    		// and add it in with the warning message reported to Eclipse.
    		if(outputLine.startsWith("---------------"))
    		{
    			if(more)
    			{
    				more = false;
    				counter = 0;
    				if(currentMessage != null)
    				{
    					// counter++;
    					// currentMessage.setText(counter + " " + moreString);
    					// messages.add(currentMessage);
    				}
    			}
    			continue;
    		}
    		
    		int fileColon = outputLine.indexOf(':');
    		String fileName;
    		if (fileColon != -1)
    		{
    			fileName = outputLine.substring(0, fileColon);
    		} else {
    			if(more)
    			{
    				counter++;
    				EscjavaMessage newMessage = (EscjavaMessage) currentMessage.clone();
    				newMessage.setText(counter + " " + outputLine);
    				messages.add(newMessage);
    			}
    				// moreString = moreString + outputLine + "\n";
    				// change \n to the java method for newline char
    			continue;
    		}
    		
    		int lineColon = outputLine.indexOf(':', fileColon + 1);
    		String lineString;
    		if (lineColon != -1)
    		{
    			lineString = outputLine.substring(fileColon + 1, lineColon);
    		} else {
				if(more)
				{
					counter++;
					EscjavaMessage newMessage = (EscjavaMessage) currentMessage.clone();
					newMessage.setText(counter + " " + outputLine);
					messages.add(newMessage);
				}
					// moreString = moreString + outputLine + "\n";
					// change \n to the java method for newline char
    			continue;
    		}
    		
    		int detailsLocation = outputLine.indexOf(':', lineColon + 1);
    		String detailsString;
    		if (detailsLocation != -1)
    		{
    			detailsString = outputLine.substring(detailsLocation + 2);
    		} else {
				if(more)
				{
					counter++;
					EscjavaMessage newMessage = (EscjavaMessage) currentMessage.clone();
					newMessage.setText(counter + " " + outputLine);
					messages.add(newMessage);
				}
					// moreString = moreString + outputLine + "\n";
					// change \n to the java method for newline char
    			continue;
    		}
    		
    		EscjavaMessage message = new EscjavaMessage(
    			Integer.parseInt(lineString),
    			0, 
    			"",
    			SeverityLevel.WARNING,
    			fileName);
    			
    		currentMessage = (EscjavaMessage) message.clone();
    		
    		currentMessage.setText(counter + " " + detailsString);
    		
    		messages.add(currentMessage);
    		
    		more = true;
    		// moreString = "";
       	}   
       	
       	return (EscjavaMessage[]) messages.toArray(new EscjavaMessage[0]);	
    }


    /**
     * Create a stripped down version of a filename.
     * @param aFileName the original filename
     * @return the filename where an initial prefix of basedir is stripped
     */
    private String getStrippedFileName(final String aFileName)
    {
        final String stripped;
        if ((mBasedir == null) || !aFileName.startsWith(mBasedir)) {
            stripped = aFileName;
        }
        else {
            // making the assumption that there is text after basedir
            final int skipSep = mBasedir.endsWith(File.separator) ? 0 : 1;
            stripped = aFileName.substring(mBasedir.length() + skipSep);
        }
        return stripped;
    }

    /** @param aBasedir the base directory to strip off in filenames */
    public void setBasedir(String aBasedir)
    {
        mBasedir = aBasedir;
    }

    /**
     * notify all listeners about the errors in a file.
     * @param aFileName the audited file
     * @param aErrors the audit errors from the file
     */
    public void fireErrors(String aFileName, EscjavaMessage[] aErrors)
    {
        final String stripped = getStrippedFileName(aFileName);
        for (int i = 0; i < aErrors.length; i++) {
            final AuditEvent evt = new AuditEvent(this, stripped, aErrors[i]);
            final Iterator it = mListeners.iterator();
            while (it.hasNext()) {
                final AuditListener listener = (AuditListener) it.next();
                listener.addError(evt);
            }
        }
    }

    /**
     * Sets the severity level.  The string should be one of the names
     * defined in the <code>SeverityLevel</code> class.
     *
     * @param aSeverity  The new severity level
     * @see SeverityLevel
     */
    public final void setSeverity(String aSeverity)
    {
        mSeverityLevel = SeverityLevel.getInstance(aSeverity);
    }
}
