/*
 * (c) Copyright 2003 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */
package edu.virginia.cs.escjava;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

/** creates markers for all checkstyle errors on the current resource.
 * Listens to checkstyle events.
 * 
 * @author Marco van Meegen
 */
public class EscjavaErrorHandler implements AuditListener {
	
	/** resource this error handler puts markers on */
	private IResource resource = null;

	/**
	 * creates an error handler for the given resource
	 * @param resource resource to put markers on
	 */
	public EscjavaErrorHandler(IResource resource) {
		this.resource = resource;
	}

	/**
	 * creates a marker with the given parameters.
	 * @param msg message
	 * @param type severity
	 * @param line line number in current resource
	 */
	void createMarker(String msg, int type, int line, String text) {
		try {
			IMarker im = resource.createMarker(EscjavaPlugin.CHECKCLIPSE_MARKER_ID);
			String[] attNames = {IMarker.MESSAGE, IMarker.SEVERITY, IMarker.LINE_NUMBER };
			Object[] attValues = {msg + "\n" + text, new Integer(type), new Integer(line)};
			im.setAttributes(attNames, attValues);
		} catch (CoreException ex) {
			log("Exception during setting marker: " + ex);
			ex.printStackTrace();
		}
	}

	/** 
	 * echoes that file is started
	 * @see com.puppycrawl.tools.checkstyle.AuditListener#fileStarted(com.puppycrawl.tools.checkstyle.AuditEvent)
	 */
	public void fileStarted(AuditEvent arg0) {
		log("checking file " + arg0.getFileName() + " ...");
	}

	/** 
	 * echoes that file is ended
	 * @see com.puppycrawl.tools.checkstyle.AuditListener#fileFinished(com.puppycrawl.tools.checkstyle.AuditEvent)
	 */
	public void fileFinished(AuditEvent arg0) {
		log("file " + arg0.getFileName() + " checked.");
	}

	/** 
	 * overridden to create error markers for all style violations
	 * @see com.puppycrawl.tools.checkstyle.AuditListener#addError(com.puppycrawl.tools.checkstyle.AuditEvent)
	 */
	public void addError(AuditEvent event) {
		if (event.getSeverityLevel() != SeverityLevel.IGNORE) {
			createMarker(event.getMessage(), adaptLevel(event.getSeverityLevel()), event.getLine(), event.getText());
			System.out.println("creating marker: " + event.getMessage() + "asdf " + event.getText());
		}
	}

	/**
	 * @param level level to adapt
	 * @return adapted level
	 */
	private int adaptLevel(SeverityLevel level) {
		if (level == SeverityLevel.ERROR) {
			// TODO maintain compatibility, because default is error, not warning
			return IMarker.SEVERITY_WARNING;
		} else if (level == SeverityLevel.INFO) {
			return IMarker.SEVERITY_INFO;
		} else if (level == SeverityLevel.WARNING) {
			return IMarker.SEVERITY_WARNING;
		}
		// all unknown levels are mapped to warning
		return IMarker.SEVERITY_WARNING;
	}

	/**
	 * logs message on console
	 * @param string string to log
	 */
	private void log(String string) {
		// TODO console !
		System.out.println(string);

	}

	/** 
	 * catch checkstyle exceptions occured and log them to console
	 * @see com.puppycrawl.tools.checkstyle.AuditListener#addException(com.puppycrawl.tools.checkstyle.AuditEvent, java.lang.Throwable)
	 */
	public void addException(AuditEvent arg0, Throwable arg1) {
		log("Checkstyle Exception occured while auditing: ");
		log(arg0.toString());
		arg1.printStackTrace();
	}

	/* (non-Javadoc)
	 * @see edu.virginia.cs.escjava.AuditListener#auditStarted(edu.virginia.cs.escjava.AuditEvent)
	 */
	public void auditStarted(AuditEvent aEvt) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see edu.virginia.cs.escjava.AuditListener#auditFinished(edu.virginia.cs.escjava.AuditEvent)
	 */
	public void auditFinished(AuditEvent aEvt) {
		// TODO Auto-generated method stub
		
	}

}
