/*
 * (c) Copyright 2003 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */
package edu.virginia.cs.escjava.preferences;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import edu.virginia.cs.escjava.EscjavaPlugin;
import edu.virginia.cs.escjava.EscjavaConfiguration;
import edu.virginia.cs.escjava.Checker;

/**
 * Preferences for Checkclipse. Can be overridden for each project
 * @author Marco van Meegen
 */

public class EscjavaPreferences extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

	/** constructor */
	public EscjavaPreferences() {
		super(GRID);
		setPreferenceStore(EscjavaPlugin.getDefault().getPreferenceStore());
		setDescription("ESC/Java Preferences");
		initializeDefaults();
	}
	/**
	 * no defaults are set
	 */
	private void initializeDefaults() {
	}

	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
				
		addField(
			new FileFieldEditor(
				EscjavaPlugin.ESCJAVA_LOCATION_PREFERENCE,
				"ESC/Java Executable Location:",
				true,
				getFieldEditorParent()));
			
	}

	/**
	 * needs to be overridden  
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		//Initialize the preference store we wish to use
		setPreferenceStore(EscjavaPlugin.getDefault().getPreferenceStore());
	}
	
	/** 
	 * invalidate checker cache if properties change
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		EscjavaConfiguration.invalidate();
	}

	/** 
	 * check if configuration is valid by creating a checker which fits
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		// first save preferences
		boolean ok = super.performOk();
		if (ok) {
			// now try to create a checker
			try {
				Checker checker = EscjavaConfiguration.getGlobalChecker();
				checker.destroy();
				rebuildAll();
			} catch (Exception ex) {
				// something went wrong, stay here
				setErrorMessage("Invalid Configuration: " + ex.getMessage());
				return false;
			}
		}
		return ok;
	}

	/** rebuilds all projects in workspace */
	private void rebuildAll() {
		boolean autoRebuild = EscjavaPlugin.getDefault().getPreferenceStore().getBoolean(EscjavaConfiguration.P_AUTO_REBUILD);
		if (autoRebuild) {
			// rebuild all
			ProgressMonitorDialog dlg = new ProgressMonitorDialog(getShell());
	
			try {
				dlg.run(true, true, new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						try {
							ResourcesPlugin.getWorkspace().build(
								IncrementalProjectBuilder.FULL_BUILD,
								monitor);
						} catch (CoreException ex) {
							ex.printStackTrace();
						}
					}
				});
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				dlg.close();
			}
		}
	}

}